//
// KBlackbox
//
// A simple game inspired by an emacs module
//
/***************************************************************************
 *   Copyright (c) 1999-2000, Robert Cimrman                               *
 *   cimrman3@students.zcu.cz                                              *
 *                                                                         *
 *   Copyright (c) 2007, Nicolas Roffet                                    *
 *   nicolas-kde@roffet.com                                                *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA               *
 ***************************************************************************/



#include <QGraphicsScene>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsLineItem>
#include <QGraphicsRectItem>
#include <QPen>


#include "kbbgraphicsitemblackbox.h"
#include "kbbscalablegraphicwidget.h"



//
// Constructor / Destructor
//

KBBGraphicsItemBlackBox::KBBGraphicsItemBlackBox( KBBScalableGraphicWidget* parent, QGraphicsScene* scene) : QGraphicsRectItem (0, scene )
{
	m_widget = parent;
	m_scene = scene;

	setPen(QPen(Qt::black));
	setBrush(QColor(65, 65, 65));
}



//
// Public
//

void KBBGraphicsItemBlackBox::setSize(const int columns, const int rows)
{
	if ((m_columns!=columns) && (m_rows!=rows)) {
		m_columns = columns;
		m_rows = rows;
		
		setRect(KBBScalableGraphicWidget::BORDER_SIZE, KBBScalableGraphicWidget::BORDER_SIZE, m_columns*KBBScalableGraphicWidget::RATIO, m_rows*KBBScalableGraphicWidget::RATIO);
	
		//remove old lines
		for (int i=0; i<m_lines.count(); i++)
			delete m_lines[i];
		m_lines.clear();
	
		// add new lines
		for (int i=1; i<m_columns; i++)
			m_lines.append(new QGraphicsLineItem( KBBScalableGraphicWidget::BORDER_SIZE + i*KBBScalableGraphicWidget::RATIO, KBBScalableGraphicWidget::BORDER_SIZE, KBBScalableGraphicWidget::BORDER_SIZE + i*KBBScalableGraphicWidget::RATIO, KBBScalableGraphicWidget::BORDER_SIZE + m_rows*KBBScalableGraphicWidget::RATIO, this, m_scene));
		for (int i=1; i<m_rows; i++)
			m_lines.append(new QGraphicsLineItem(  KBBScalableGraphicWidget::BORDER_SIZE, KBBScalableGraphicWidget::BORDER_SIZE + i*KBBScalableGraphicWidget::RATIO, KBBScalableGraphicWidget::BORDER_SIZE + m_columns*KBBScalableGraphicWidget::RATIO,  KBBScalableGraphicWidget::BORDER_SIZE + i*KBBScalableGraphicWidget::RATIO, this, m_scene));
		
		// set line style
		for (int i=0; i<m_lines.count(); i++) {
			QPen pen(Qt::DashLine);
			pen.setColor(Qt::black);
			pen.setWidth(0);
			m_lines[i]->setPen(pen);
		}
	}
}



//
// Private
//

void KBBGraphicsItemBlackBox::mousePressEvent (QGraphicsSceneMouseEvent* event)
{
	int x = (static_cast<int>(event->pos().x()) - KBBScalableGraphicWidget::BORDER_SIZE);
	int y = (static_cast<int>(event->pos().y()) - KBBScalableGraphicWidget::BORDER_SIZE);
	x /= KBBScalableGraphicWidget::RATIO;
	y /= KBBScalableGraphicWidget::RATIO;
	m_widget->clickBlackBox(x + y*m_columns);
}
